# Generated by Django 5.0.10 on 2025-01-31 09:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0006_notification_created'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationtype',
            name='resource_button_label',
            field=models.CharField(default='Open Resource', max_length=128),
            preserve_default=False,
        ),
    ]
