# Generated by Django 4.2.6 on 2023-11-16 10:09

import colorfield.fields
import django_better_admin_arrayfield.models.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [("wbcore", "0013_delete_colorgradient")]

    state_operations = [
        migrations.CreateModel(
            name="ColorGradient",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(default="", max_length=128)),
                (
                    "colors",
                    django_better_admin_arrayfield.models.fields.ArrayField(
                        base_field=colorfield.fields.ColorField(
                            default="#FF0000", image_field=None, max_length=18, samples=None
                        ),
                        default=list,
                        size=None,
                    ),
                ),
            ],
        ),
    ]

    operations = state_operations
