{% extends "email_base_template.html" %}
{% load i18n %}
{% block content %}
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <p>
            <b>{% translate "Confirm Your Email" %}</b>
        </p>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <p>{% translate "Please click on the link to confirm your registration." %}</p>
    </td>

    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <table cellspacing="0" cellpadding="0">
            <tr>
                <td style="border-radius: 10px;" bgcolor="#f76f5b">
                    <a class="link" href="{{url}}" target="_blank" style="padding: 8px 12px; border: 1px solid #ED2939; border-radius: 10px;font-family: Helvetica, Arial, sans-serif;font-size: 14px; color: #ffffff;text-decoration: none;font-weight:bold;display: inline-block;">
                        {% translate "Confirm Email" %}
                    </a>
                </td>
            </tr>
        </table>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>

{% endblock content %}
