from . import AbstractBackend


class IconBackend(AbstractBackend):
    """
    List of Icon supported from the Google Icon Material.

    They can be added and search through https://fonts.google.com/icons?icon.set=Material+Icons
    """

    @classmethod
    @property
    def fallback_icon(cls):
        return "broken_image"

    ACCOUNT_BALANCE = "account_balance"
    ADD = "add_circle"
    APPROVE = "thumb_up_alt"
    BANK = "account_balance"
    BIRTHDAY = "cake"
    BOOKMARK = "bookmark_border"
    CALENDAR = "calendar_month"
    CHART_AREA = "stacked_line_chart"
    CHART_BARS_HORIZONTAL = "align_horizontal_center"
    CHART_BARS_VERTICAL = "bar_chart"
    CHART_BARS_VERTICAL_INCREASING = "signal_cellular_alt"
    CHART_LINE = "show_chart"
    CHART_PIE = "pie_chart"
    CHART_PYRAMID = "bar_chart"
    CHART_SWITCHES = "candlestick_chart"
    CHART_WATERFALL = "waterfall_chart"
    CHART_MULTILINE = "ssid_chart"
    CLIPBOARD = "assignment"
    CONFIGURE = "tune"
    CONFIRM = "check_circle"
    COPY = "content_copy"
    CURRENCY_EXCHANGE = "currency_exchange"
    DASHBOARD = "dashboard"
    DATA_GRID = "analytics"
    DATA_LIST = "table_view"
    DATA_EXPLORATION = "query_stats"
    DAY_OFF = "event_busy"
    DEAL = "handshake"
    DEAL_MONEY = "request_quote"
    DECISION_STEP = "alt_route"
    DELETE = "delete"
    DENY = "thumb_down_alt"
    DISABLED = "block"
    DOCUMENT = "description"
    DOCUMENT_IN_PROGRESS = "history_edu"
    DOCUMENT_PRIVATE = "assignment_late"
    DOCUMENT_WITH_DOLLAR = "request_page"
    DOCUMENT_WITH_ATTACHMENT = "file_present"
    DOLLAR = "attach_money"
    DOWNLOAD = "file_download"
    EDIT = "edit"
    END = "last_page"
    ENUMERATION = "format_list_numbered"
    EURO = "euro"
    EVENT = "event"
    EVENT_ACCEPTED = "event_available"
    EXPEND = "fullscreen"
    FAVORITE = "star"
    FEEDBACK = "feedback"
    FOLDERS_ADD = "create_new_folder"
    FOLDERS_MONEY = "folder_special"
    FOLDERS_OPEN = "folder_open"
    GENERATE_NEXT = "next_plan"
    GOAL = "sports_soccer"
    GROUPS = "groups"
    HOME = "home"
    HISTORY = "history"
    IGNORE = "visibility_off"
    IMPORT_EXPORT = "import_export"
    INFO = "info"
    INVERSE = "swap_horiz"
    JOIN_STEP = "merge"
    LINK = "link"
    LOCATION = "location_on"
    LOCK = "lock"
    LUNCH = "restaurant"
    MAIL = "email"
    MAIL_MONEY = "attach_email"
    MAIL_OPEN = "mark_email_read"
    MAIL_POSTED = "forward_to_inbox"
    MERGE = "merge"
    NEWSPAPER = "newspaper"
    NEXT = "fast_forward"
    NOTEBOOK = "notes"
    OTHER = "more_horiz"
    PENDING = "pending"
    PEOPLE = "people_alt"
    PERSON = "person"
    PERSON_ADD = "person_add"
    PHONE = "smartphone"
    PHONE_ADD = "add_ic_call"
    PREVIOUS = "fast_rewind"
    PUBLIC = "public"
    QUESTION = "question_mark"
    REDO = "redo"
    REFRESH = "refresh"
    REGENERATE = "restart_alt"
    REJECT = "cancel"
    REPEAT = "repeat"
    REPLACE = "published_with_changes"
    REVIEW = "rate_review"
    RUNNING = "running_with_errors"
    SAVE = "save"
    SCHEDULE = "schedule"
    SCRIPT = "terminal"
    SEND = "send"
    SEND_LATER = "schedule_send"
    SETTINGS = "settings"
    SHARE = "share"
    SHRINK = "fullscreen_exit"
    SICK_LEAVE = "sick"
    SPLIT = "call_split"
    START = "start"
    STATS = "query_stats"
    SUPERVISE = "supervised_user_circle"
    SYNCHRONIZE = "sync"
    TABLE = "table_chart"
    TIME_UP = "hourglass_bottom"
    TRADE = "request_quote"
    TRAINING = "school"
    UNDO = "restore"
    UNFILTER = "filter_alt_off"
    UNFOLD = "arrow_drop_down"
    UNLINK = "link_off"
    UNLOCK = "lock_open"
    UPLOAD = "file_upload"
    VACATION = "holiday_village"
    VIEW = "visibility"
    WARNING = "warning"
    WORK = "card_travel"
