# Generated by Django 4.1.10 on 2023-08-08 13:31

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("io", "0001_initial_squashed"),
    ]

    operations = [
        migrations.AddField(
            model_name="importsource",
            name="creator",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Creator",
            ),
        ),
    ]
