{% extends 'base.html' %}
{% load i18n %}
{% block content %}
<div class="container-500">
    <h1>{% translate "Password Reset" %}</h1>
</div>
<p>{% translate "Please enter the email address you used to register
    with us and we will send you a link to reset your password." %}</p>
    <form action="" method="POST" class="text-center">
        {% csrf_token %}
        {{form}}
        <input type="submit" value="Send email" /><br>
      </form><br/>

{% endblock%}
