# Generated by Django 4.1.9 on 2023-06-13 09:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("agenda", "0007_alter_calendaritem_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="calendaritem",
            name="item_type",
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name="Type"),
        ),
    ]
