{% extends "wbcore/email_base_template.html" %}
{% load i18n %}
{% block body %}
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <p>
            <b>{{ title }}</b>
        </p>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        {{ message | safe}}
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
{% if notification_share_url %}
<tr class="content-row">
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
        <table cellspacing="0" cellpadding="0">
            <tr>
                <td style="border-radius: 10px;" bgcolor="#f76f5b">
                    <a class="link" href="{{ notification_share_url }}" target="_blank" style="padding: 8px 12px; border: 1px solid #ED2939; border-radius: 10px;font-family: Helvetica, Arial, sans-serif;font-size: 14px; color: #ffffff;text-decoration: none;font-weight:bold;display: inline-block;">
                        {% translate "Open" %}
                    </a>
                </td>
            </tr>
        </table>
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
<tr>
    <td style="background-color: #eaf2ff;"></td>
    <td class="body-content">
    </td>
    <td style="background-color: #eaf2ff;"></td>
</tr>
{% endif %}
{% endblock %}
