# Generated by Django 5.0.12 on 2025-02-28 10:21

import modeltrans.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0010_remove_addresscontact_city'),
    ]

    operations = [
        migrations.AddField(
            model_name='person',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AddField(
            model_name='person',
            name='i18n',
            field=modeltrans.fields.TranslationField(fields=['description'], required_languages=(), virtual_fields=True),
        ),
    ]
