# Generated by Django 4.1.10 on 2023-09-12 09:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("workflow", "0002_alter_step_step_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="condition",
            name="attribute_name",
            field=models.CharField(
                help_text="The name of the attribute on the attached instance. Its value will be evaluated against the condition.",
                max_length=64,
                verbose_name="Attribute Name",
            ),
        ),
        migrations.AlterField(
            model_name="condition",
            name="expected_value",
            field=models.CharField(
                help_text="The expected value of the attribute that will satisfy the condition.",
                max_length=64,
                verbose_name="Expected Value",
            ),
        ),
        migrations.AlterField(
            model_name="process",
            name="content_type",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="contenttypes.contenttype",
                verbose_name="Attached Model",
            ),
        ),
        migrations.AlterField(
            model_name="step",
            name="status",
            field=models.CharField(
                blank=True,
                help_text="The status that will be set in the attached instance's status field upon transitioning to this step. Only applicable if attached model is set.",
                max_length=64,
                null=True,
                verbose_name="Status",
            ),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="single_instance_execution",
            field=models.BooleanField(
                default=False,
                help_text="Indicates wether the workflow is restricted to only one running workflow at a time. If an instance is attached to the workflow this will restrict the workflow execution per model instance.",
                verbose_name="Single Instance Execution",
            ),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="status_field",
            field=models.CharField(
                blank=True,
                help_text="The status field name in the attached model that needs to be updated by the workflow",
                max_length=64,
                null=True,
                verbose_name="Status Field",
            ),
        ),
    ]
