# Generated by Django 4.1.8 on 2023-04-13 10:33

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0001_initial"),
        ("authentication", "0001_initial_squashed"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="profile",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="user_account",
                to="directory.person",
            ),
        )
    ]
