{% extends 'base.html' %}
{% load i18n %}
{% block content%}
<div class="container-500">
  <h1>{% translate "Register Account" %}</h1>
</div>
<h1>{{ title }}</h1>
<p>{{ message }}</p>
<div class="text-center">
  <a href="{{redirect_url}}">{{ button_title }}</a>
</div>
{% endblock%}
