# Generated by Django 5.0.14 on 2025-05-05 09:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('directory', '0012_alter_person_managers'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='clientmanagerrelationship',
            options={'permissions': (('administrate_clientmanagerrelationship', 'Can administrate Client Manager Relationship'),), 'verbose_name': 'Client Manager Relationship', 'verbose_name_plural': 'Client Manager Relationships'},
        ),
    ]
