# Generated by Django 5.0.12 on 2025-02-18 09:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('io', '0007_alter_exportsource_query_params'),
    ]

    operations = [
        migrations.AddField(
            model_name='importsource',
            name='resource_kwargs',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
